/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Collection;
import org.openide.util.Lookup;
import org.openide.util.Lookup.Result;
import org.openide.util.Lookup.Template;
import org.openide.util.LookupListener;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class LazyLookup extends Lookup
  {
    @Override @CheckForNull
    public synchronized <T> T lookup (final @Nonnull Class<T> clazz)
      {
        final Collection<? extends T> instances = lookup(new Template<T>(clazz)).allInstances();
        return instances.isEmpty() ? null : instances.iterator().next();
      }

    @Override @CheckForNull
    public <T> Result<T> lookup (final @Nonnull Template<T> template)
      {
        return new Result<T>()
          {
            @Override
            public void addLookupListener (LookupListener l)
              {
              }

            @Override
            public void removeLookupListener (LookupListener l)
              {
              }

            @Override @Nonnull
            public Collection<? extends T> allInstances()
              {
                return (Collection<? extends T>)computeInstances(template);
              }
          };
      }

    @Nonnull
    protected abstract Collection<?> computeInstances (@Nonnull Template<?> template);
  }